@@ -28,7 +28,7 @@ def make_pod(
     port,
     image,
     image_pull_policy,
-    image_pull_secret=None,
+    image_pull_secrets=None,
     node_selector=None,
     run_as_uid=None,
     run_as_gid=None,
@@ -80,9 +80,11 @@ def make_pod(
         Image pull policy - one of 'Always', 'IfNotPresent' or 'Never'. Decides
         when kubernetes will check for a newer version of image and pull it when
         running a pod.
-    image_pull_secret:
-        Image pull secret - Default is None -- set to your secret name to pull
-        from private docker registry.
+    image_pull_secrets:
+        Image pull secrets - a list of references to Kubernetes Secret resources
+        with credentials to pull images from image registries. This list can
+        either have strings in it or objects with the string value nested under
+        a name field.
     port:
         Port the notebook server is going to be listening on
     cmd:
@@ -230,11 +232,16 @@ def make_pod(
     pod.spec = V1PodSpec(containers=[])
     pod.spec.restart_policy = 'OnFailure'
 
-    if image_pull_secret is not None:
-        pod.spec.image_pull_secrets = []
-        image_secret = V1LocalObjectReference()
-        image_secret.name = image_pull_secret
-        pod.spec.image_pull_secrets.append(image_secret)
+    if image_pull_secrets is not None:
+        # image_pull_secrets as received by the make_pod function should always
+        # be a list, but it is allowed to have "a-string" elements or {"name":
+        # "a-string"} elements.
+        pod.spec.image_pull_secrets = [
+            V1LocalObjectReference(name=secret_ref)
+            if type(secret_ref) == str else
+            get_k8s_model(V1LocalObjectReference, secret_ref)
+            for secret_ref in image_pull_secrets
+        ]
 
     if node_selector:
         pod.spec.node_selector = node_selector
@@ -274,12 +281,24 @@ def make_pod(
     if all([e is None for e in container_security_context.to_dict().values()]):
         container_security_context = None
 
+    # Transform a dict into valid Kubernetes EnvVar Python representations. This
+    # representation shall always have a "name" field as well as either a
+    # "value" field or "value_from" field. For examples see the
+    # test_make_pod_with_env function.
+    prepared_env = []
+    for k, v in (env or {}).items():
+        if type(v) == dict:
+            if not "name" in v:
+                v["name"] = k
+            prepared_env.append(get_k8s_model(V1EnvVar, v))
+        else:
+            prepared_env.append(V1EnvVar(name=k, value=v))
     notebook_container = V1Container(
         name='notebook',
         image=image,
         working_dir=working_dir,
         ports=[V1ContainerPort(name='notebook-port', container_port=port)],
-        env=[V1EnvVar(k, v) for k, v in (env or {}).items()],
+        env=prepared_env,
         args=cmd,
         image_pull_policy=image_pull_policy,
         lifecycle=lifecycle_hooks,
@@ -463,6 +482,7 @@ def make_ingress(
         name,
         routespec,
         target,
+        labels,
         data
 ):
     """
@@ -497,11 +517,7 @@ def make_ingress(
             'hub.jupyter.org/proxy-routespec': routespec,
             'hub.jupyter.org/proxy-target': target
         },
-        labels={
-            'heritage': 'jupyterhub',
-            'component': 'singleuser-server',
-            'hub.jupyter.org/proxy-route': 'true'
-        }
+        labels=labels,
     )
 
     if routespec.startswith('/'):
@@ -515,7 +531,7 @@ def make_ingress(
     target_ip = target_parts.hostname
     target_port = target_parts.port
 
-    target_is_ip = re.match('^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$', target_ip) is not None
+    target_is_ip = re.match(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$', target_ip) is not None
 
     # Make endpoint object
     if target_is_ip:
